﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;

namespace RoboRace_15
{
    public partial class Form1 : Form
    {
        string server, prikazy, puvodni_pozice;
        char[,] pole;
        Lerak robot;
        string id;
        public Form1(string url, int port)
        {
            server = "http://" + url + ":" + port.ToString();
            pole = new char[20,20];
            try
            {
                WebRequest wr = WebRequest.Create(server + "/start");
                using (StreamReader sr = new StreamReader(wr.GetResponse().GetResponseStream()))
                {
                    id = sr.ReadLine();
                    for (int y = 0; y < 20; y++)
                    {
                        string radek = sr.ReadLine();
                        for (int x = 0; x < 20; x++)
                            pole[x, y] = radek[x];
                    }
                    nastavit_pozici(sr.ReadLine());
                    prikazy = sr.ReadLine();
                }
            }
            catch (Exception e) { MessageBox.Show("Nepodařilo se připojit k serveru\n" + e.Message); Application.Exit(); }
            InitializeComponent();
            nastavit_prikazy();
            Text = "ID: " + id;
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            int sirka = ((Panel)sender).Width / 20;
            int vyska = ((Panel)sender).Height / 20;
            Pen pero = new Pen(Color.Black);
            for (int i = 1; i <= 20; i++)
            {
                g.DrawLine(pero, new Point(i * sirka, 0), new Point(i * sirka, ((Panel)sender).Height));
                g.DrawLine(pero, new Point(0,i * vyska), new Point(((Panel)sender).Width,i * vyska));
            }
            for (int x = 0; x < pole.GetLength(0); x++)
                for (int y = 0; y < pole.GetLength(1); y++)
                    g.DrawString(pole[x, y].ToString(), new Font("Verdana", 8), Brushes.Navy, new Point(x * sirka, y*vyska));
            g.DrawString(orientace_sipka(robot.smer), new Font("Verdana", 8), Brushes.Red, new Point(robot.x * sirka, robot.y * vyska));
        }

        //převedení N/D/L/R na ↑/↓/←/→
        string orientace_sipka(smer orientace)
        {
            switch (orientace)
            {
                case smer.N: return "↑";
                case smer.D: return "↓";
                case smer.R: return "→";
                case smer.L: return "←";
                default: return "?";
            }
        }

        private void Form1_ResizeEnd(object sender, EventArgs e)
        {
            panel1.Refresh();
        }

        //reset zadaných příkazů
        private void button1_Click(object sender, EventArgs e)
        {
            nastavit_prikazy();
            label2.ResetText();
            nastavit_pozici(puvodni_pozice);
            panel1.Refresh();
        }

        private void checkedListBox1_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if(e.NewValue == CheckState.Checked)
                label2.Text += checkedListBox1.Items[e.Index];
            //nelze "odškrtnout" příkaz
            else
                e.NewValue = CheckState.Checked;
        }

        //nastavení seznamu možných příkazů
        void nastavit_prikazy()
        {
            checkedListBox1.Items.Clear();
            foreach (char ch in prikazy)
                checkedListBox1.Items.Add(ch);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            foreach (char ch in label2.Text)
            {
                if (!proved_prikaz(ch))
                    break;
            }
            panel1.Refresh();
        }

        private bool proved_prikaz(char ch)
        {
            bool uspesny = true;
            switch(ch)
            {
                case 'L': robot.L(); break;
                case 'R': robot.R(); break;
                case '3': uspesny = vpred(3); break;
                case '2': uspesny = vpred(2); break;
                case '1': uspesny = vpred(1); break;
                case 'Z': uspesny = zpet(); break;
            }
            if (!uspesny) return false;

            if (pole[robot.x, robot.y] != '.' && pole[robot.x, robot.y] != '$')
            {
                smer puvodni_s = robot.smer;
                switch (pole[robot.x, robot.y])
                {
                    case '<': robot.smer = smer.L; break;
                    case '^': robot.smer = smer.N; break;
                    case '>': robot.smer = smer.R; break;
                    case 'v': robot.smer = smer.D; break;
                }
                vpred(1);
                robot.smer = puvodni_s;
            }
            return true;
        }

        private bool zpet()
        {
            int puvodni_x = robot.x;
            int puvodni_y = robot.y;
            switch (robot.smer)
            {
                case smer.N: robot.y++; break;
                case smer.L: robot.x++; break;
                case smer.D: robot.y--; break;
                case smer.R: robot.x--; break;
            }
            if(kontrola())
                if (pole[robot.x, robot.y] == 'X')
                {
                    robot.x = puvodni_x;
                    robot.y = puvodni_y;
                }
            return kontrola();
        }

        private bool vpred(int pocet)
        {
            int puvodni_x = robot.x;
            int puvodni_y = robot.y;
            for (int i = 0; i < pocet; i++)
            {
                switch (robot.smer)
                {
                    case smer.N: robot.y--; break;
                    case smer.L: robot.x--; break;
                    case smer.D: robot.y++; break;
                    case smer.R: robot.x++; break;
                }
                if (!kontrola())
                    return false;
                if (pole[robot.x, robot.y] == 'X')
                {
                    robot.x = puvodni_x;
                    robot.y = puvodni_y;
                    return true;
                }
                if(pole[robot.x, robot.y] == '$')
                    return true;
            }
            return kontrola();
        }

        private bool kontrola()
        {
            if (robot.x < 0 || robot.x >= 20 || robot.y < 0 || robot.y >= 20)
            {
                MessageBox.Show("Robot spadl z okraje");
                return false;
            }
            else if (pole[robot.x, robot.y] == '#')
            {
                MessageBox.Show("Lerak zahučel do díry");
                return false;
            }
            else return true;
        }

        //odeslání tahu na servr
        private void button3_Click(object sender, EventArgs e)
        {
            WebRequest wr = WebRequest.Create(server + "/kolo?id=" + id + "&pohyby=" + label2.Text);
            string test = "";
            using (StreamReader sr = new StreamReader(wr.GetResponse().GetResponseStream()))
            {
                string vystup = sr.ReadLine();
                if (vystup != id)
                {
                    MessageBox.Show(vystup);
                    Application.Exit();
                    return;
                }
                //přeskočení ID a mapy, jelikož se nemění
                for (int i = 0; i < 20; i++)
                    test += sr.ReadLine();
                nastavit_pozici(sr.ReadLine());
                prikazy = sr.ReadLine();
                nastavit_prikazy();
            }
            panel1.Refresh();
            label2.ResetText();
        }

        private void nastavit_pozici(string poz)
        {
            puvodni_pozice = poz;
            string[] pozice = puvodni_pozice.Split(new char[] { ' ' });
            smer s;
            switch (pozice[2])
            {
                case "N": s = smer.N; break;
                case "L": s = smer.L; break;
                case "D": s = smer.D; break;
                case "R": s = smer.R; break;
                default: MessageBox.Show("Špatný formát směru"); s = smer.N; break;
            }
            robot = new Lerak(Convert.ToInt32(pozice[0]) - 1, Convert.ToInt32(pozice[1]) - 1, s);
        }
    }

    enum smer
    { N = 0, R = 1, D = 2, L = 3 }
}